; -----------------------------------------------------------------------------
; Calcula la posición, rotación y dirección de la bola para pintarla.
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
MoveBall:
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $80                     ; Comprueba la dirección vertical
jr      nz, moveBall_down       ; Si el bit 7 está a uno, va hacia abajo

moveBall_up:
; La bola va hacia arriba
ld      hl, (ballPos)           ; Carga la posición de la bola en HL
ld      a, BALL_TOP             ; Carga en A el margen superior
call    CheckTop                ; Evalúa si se ha alcanzado el margen superior
jr      z, moveBall_upChg       ; Si se ha alcanzado salta
call    PreviousScan            ; Obtiene el scanline anterior a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_upChg:
; La bola va hacia arriba, pero ha llegado al tope y cambia de dirección
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
or      $80                     ; Pone la dirección vertical hacia abajo
ld      (ballSetting), a        ; Carga en memoria la nueva dirección de la bola
call    NextScan                ; Obtiene el scanline siguiente a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_down:
; La bola va hacia abajo
ld      hl, (ballPos)           ; Carga la posición de la bola en HL
ld      a, BALL_BOTTOM          ; Carga en A el margen superior
call    CheckBottom             ; Evalúa si se ha alcanzado el margen superior
jr      z, moveBall_downChg     ; Si se ha alcanzado salta
call    NextScan                ; Obtiene el scanline siguiente a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_downChg:
; La bola va hacia abajo, pero ha llegado al tope y cambia de dirección
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $7f                     ; Pone la dirección vertical hacia arriba
ld      (ballSetting), a        ; Carga en memoria la nueva dirección de la bola
call    PreviousScan            ; Obtiene el scanline anterior a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola

moveBall_x:
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $40                     ; Comprueba la dirección horizontal
jr      nz, moveBall_left       ; Si el bit 6 está a uno, va hacia la izquierda

moveBall_right:
; La bola va hacia la derecha
ld      a, (ballRotation)       ; Carga la rotación actual de la bola
cp      $08                     ; Comprueba si ya está en la última rotación
jr      z, moveBall_rightLast   ; Si está en la última rotación salta
inc     a                       ; Incrementa la rotación 
ld      (ballRotation), a       ; La carga en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_rightLast:
; Está en la última rotación
; Si no ha llegado al límite derecho pone la rotación a 1 
; y pone la bola en la siguiente columna
ld      a, (ballPos)            ; Carga la línea y columna de la bola en A
and     $1f                     ; Se queda solo con la columna
cp      MARGIN_RIGHT            ; Lo comprara con el límite derecho
jr      z, moveBall_rightChg    ; Si lo ha alcanzado salta
ld      hl, ballPos             ; Carga la dirección de la posición de la bola en HL
inc     (hl)                    ; Incrementa la columna
ld      a, $01                  ; Pone la rotación a 1
ld      (ballRotation), a       ; Carga el valor en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_rightChg:
; Ha llegado al límite derecho
; Pone la rotación a -1 y cambia la dirección horizontal de la bola
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
or      $40                     ; Pone la dirección horizontal hacia la izquierda
ld      (ballSetting), a        ; Carga la nueva dirección de la bola en memoria
ld      a, $ff                  ; Carga -1 en A
ld      (ballRotation), a       ; Lo carga en memoria Rotación = -1
jr      moveBall_end            ; Fin de la rutina

moveBall_left:
; La bola va hacia la izquierda
ld      a, (ballRotation)       ; Carga la rotación actual de la bola
cp      $f8                     ; Comprueba si ya está en la última rotación
jr      z, moveBall_leftLast    ; Si está en la última rotación salta
dec     a                       ; Decrementa la rotación 
ld      (ballRotation), a       ; La carga en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_leftLast:
; Esta en la última rotación
; Si no ha llegado al límite izquierdo pone la rotación a -1 
; y pone la bola en la columna anterior
ld      a, (ballPos)            ; Carga la línea y columna en A
and     $1f                     ; Se queda solo con la columna
cp      MARGIN_LEFT             ; Lo comprara con el límite izquierdo
jr      z, moveBall_leftChg     ; Si lo ha alcanzado salta
ld      hl, ballPos             ; Carga la dirección de la posición de la bola en HL
dec     (hl)                    ; Pasa a la columna anterior
ld      a, $ff                  ; Pone la rotación a -1
ld      (ballRotation), a       ; Carga el valor en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_leftChg:
; Ha llegado al límite izquierdo
; Pone la rotación a 1 y cambia la dirección
ld      a, $01                  ; Carga la posición de la bola en HL
ld      (ballRotation), a       ; Carga el valor en memoria Rotación = 1
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $bf                     ; Pone la dirección horizontal hacia la derecha
ld      (ballSetting), a        ; Carga la nueva dirección de la bola en memoria

moveBall_end:
ret

; -----------------------------------------------------------------------------
; Calcula la posición de las palas para moverlas.
; Entrada:  D -> Pulsaciones de los controles
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
MovePaddle:
bit     $00, d              ; Evalúa si se ha pulsado la A
jr      z, movePaddle_1Down ; Si no se ha pulsado salta
ld      hl, (paddle1pos)    ; Carga en HL la posición de la pala 1
ld      a, PADDLE_TOP       ; Carga en A el margen superior
call    CheckTop            ; Evalúa si se ha alcanzado el margen superior
jr      z, movePaddle_2Up   ; Si se ha alcanzado, salta
call    PreviousScan        ; Obtiene el scanline anterior a la posición de la pala 1
ld      (paddle1pos), hl    ; Carga en memoria la nueva posición de la pala 1
jr      movePaddle_2Up      ; Salta

movePaddle_1Down:
bit     $01, d              ; Evalúa si se ha pulsado la Z               
jr      z, movePaddle_2Up   ; Si no se ha pulsado salta
ld      hl, (paddle1pos)    ; Carga en HL la posición de la pala 1
ld      a, PADDLE_BOTTOM    ; Carga en A el margen inferior
call    CheckBottom         ; Evalúa si se ha alcanzado el margen inferior
jr      z, movePaddle_2Up   ; Si se ha alcanzado, salta
call    NextScan            ; Obtiene el scanline siguiente a la posición de la pala 1
ld      (paddle1pos), hl    ; Carga en memoria la nueva posición de la pala 1

movePaddle_2Up:
bit     $02, d              ; Evalúa si se ha pulsado el 0
jr      z, movePaddle_2Down ; Si no se ha pulsado salta
ld      hl, (paddle2pos)    ; Carga en HL la posición de la pala 2
ld      a, PADDLE_TOP       ; Carga en A el margen superior
call    CheckTop            ; Evalúa si se ha alcanzado el margen superior
jr      z, movePaddle_End   ; Si se ha alcanzado, salta
call    PreviousScan        ; Obtiene el scanline anterior a la posición de la pala 2
ld      (paddle2pos), hl    ; Carga en memoria la nueva posición de la pala 2
jr      movePaddle_End      ; Salta

movePaddle_2Down:
bit     $03, d              ; Evalúa si se ha pulsado la O
jr      z, movePaddle_End   ; Si no se ha pulsado salta
ld      hl, (paddle2pos)    ; Carga en HL la posición de la pala 2
ld      a, PADDLE_BOTTOM    ; Carga en A el margen inferior
call    CheckBottom         ; Evalúa si se ha alcanzado el margen inferior
jr      z, movePaddle_End   ; Si se ha alcanzado, salta
call    NextScan            ; Obtiene el scanline siguiente a la posición de la pala 2
ld      (paddle2pos), hl    ; Carga en memoria la nueva posición de la pala 2

movePaddle_End:
ret